function [A, logl, gamma] = hmm_mest (X, st, A_prev, mu, Sigma, QUIET)

%hmm_mest  Reestimates transition parameters for multiple observation sequences
%          (left-to-right HMM).
%	Use: [A, logl, gamma] = hmm_mest(X, st, A_prev, mu, Sigma). The
%	posterior probabilities gamma are usefull for reestimating the
%	gaussian parameters mu and Sigma.

% H2M Toolbox, $Revision: 2.3 $        
% Olivier Capp, 22/03/96 - $Date: 2004/11/03 14:19:34 $
% http://www.tsi.enst.fr/~cappe/h2m/

% Check input data and dimensions
error(nargchk(5, 6, nargin));
if (nargin < 6)
  QUIET = 0;
end
N = length(A_prev(1,:));
pi0 = [1 zeros(1,N-1)];
[N, p, DIAG_COV] = hmm_chk(A_prev, pi0, mu, Sigma);
[KT, nc] = size(X);
if (nc ~= p)
  error('Dimension of the observation vectors is incorrect.');
end
K = length(st);
if ((st(1) ~= 1) | (st(K) >= KT))
  error('Check the limits of the observation sequences.');
end

if (~QUIET)
  fprintf(1, 'Reestimating transition parameters..'); time = cputime;
end
% Force A to be sparse
A_prev = sparse(A_prev);
A = sparse(zeros(N));
gamma = zeros(KT, N);
den = 0;
logl = 0;
for i=1:K
  if (~QUIET)
    if (rem(i,5) == 1)
      fprintf(1, '.');
    end
  end
  % Limits of the observation sequence
  first = st(i);
  if (i == K)
    last = KT;
  else
    last = st(i+1)-1;
  end
  T = last - first + 1;
  [alpha, beta, logl_tmp, dens] = hmm_fb(X(first:last,:), A_prev, pi0, mu, Sigma, 1);
  % Compute log-likelihood
  logl = logl + logl_tmp;
  % Reestimate transition parameters
  % Numerator
  for j=1:T-1
    A_tmp = (alpha(j,:)' * (dens(j+1,:) .* beta(j+1,:))) .* A_prev;
    A_tmp = A_tmp /sum(sum(A_tmp));
    A = A + A_tmp;
  end
  % Denominator
  % Memorize state probabilities
  gamma(first:last,:) = alpha .* beta;
  gamma(first:last,:) = gamma(first:last,:) ./ (sum(gamma(first:last,:)')' * ones(1,N));
  den = den + sum(gamma(first:first+T-2,:));
end
% Normalize transition matrix
A = A ./ (den' * ones(1,N));
if (~QUIET)
  time = cputime - time; fprintf(1, ' (%.2f s)\n', time);
end
