function [A_, mu_, Sigma_, logl] = lrhmm (X, st, A_ini, mu_ini, Sigma_ini, n_it)

%lrhmm     Performs multiple iterations of the EM algorithm for a left-right
%	   model (with multiple training sequences).
%	   This is just a call to the lower-level functions.
%       Use: [A_,mu_,Sigma_,logl] = lrhmm(X,st,A_ini,mu_ini,Sigma_ini,n_it)

% H2M Toolbox, $Revision: 2.3 $        
% Olivier Capp, 07/02/97 - $Date: 2004/11/03 14:19:34 $
% http://www.tsi.enst.fr/~cappe/h2m/

% Turn verbose mode off
QUIET = 1;
% Check input data and dimensions
error(nargchk(6, 6, nargin));
% Check that the initialization value are correct
[N, p, DIAG_COV] = hmm_chk(A_ini, [1 zeros(1, length(A_ini(1,:))-1)], ...
  mu_ini, Sigma_ini);
% Use supplied values as initialization
A = A_ini;
mu = mu_ini;
Sigma = Sigma_ini;

logl = zeros(1,n_it);
for i = 1:n_it
  [A, logl(i), gamma] = hmm_mest(X, st, A, mu, Sigma, QUIET);
  [mu, Sigma] = mix_par(X, gamma, DIAG_COV, QUIET);
  fprintf(1, 'Iteration %d:\t%.3f\n', i, logl(i));
end
