function [gamma, logl] = mix_postl (X, w, mu, Sigma, QUIET)

%mix_postl Alternative to mix_post which uses logarithmic computation
%          to avoid underflows (useful in very large dimensional models).
%	Use: [gamma,logl] = mix_post (X,w,mu,Sigma) returns the a posteriori
%	probabilities. logl is the log-likehood of X.

% H2M Toolbox, $Revision: 2.2 $        
% Olivier Capp, 1995 - $Date: 2006/06/13 11:47:59 $
% http://www.tsi.enst.fr/~cappe/h2m/

% Max dynamic in log
LOG_MAX = 500;

% Input args.
error(nargchk(4, 5, nargin));
if (nargin < 5)
  QUIET = 0;
end
[N, p, DIAG_COV] = mix_chk(w, mu, Sigma);
[T, Nc] = size(X);
if (Nc ~= p)
  error('Observation vectors have an incorrect dimension.');
end

% Compute log density values
gamma = gauslogv(X, mu, Sigma, QUIET);

% A posteriori probabilities
if (~QUIET)
  fprintf(1, 'Computing a posteriori probabilities...'); time = cputime;
end
% Convert to linear scale taking care of under of overflows
gmax = max(gamma')'-LOG_MAX;
logl_corr = sum(gmax);
gamma = exp(gamma - (gmax*ones(1,N)));
gamma = gamma .* (ones(T,1) * w);
if (nargout > 1)
  % Compute log-likelihood (with correction factor)
  logl = sum(log(sum(gamma'))) + logl_corr;
end
gamma = gamma ./ ((sum(gamma'))' * ones(1,N));
if (~QUIET)
  time = cputime - time; fprintf(1, ' (%.2f s)\n',time);
end
